% synthesise combination of music and background sinusoid
% use adaptive filter to remove background

% get piece of music
[t, x, fs] = get_sound('handel');
nt = length(t);

% sinusoidal noise source
f = 440;   % (Hz) frequency (middle A)
phi = pi/3; % phase (rad)

n = sin(2*pi*f*t+phi);

% edit to similar length and normalise

% make contaminated signal
y = x+n;

fig(1, 6, 4)
subplot(2, 1, 1)
plot(t, y, t, x)
subplot(2, 1, 2)
plot(t, y, t, x)
xlim([1.5 1.6])
legend('signal+noise', 'signal')


%% Noise cancellation

% noise predictor signals (quadrature pair)
p1 = sin(2*pi*440*t);
p2 = cos(2*pi*440*t);

% set initial weights to zero
w1 = 0; w2 = 0;

% assign storage
z  = NaN(1, nt);  % cerebellar output
xe = NaN(1, nt);  % estimate of signal x
e  = NaN(1, nt);  % error in estimate

% Train adaptive filter sequentially rather than batch
beta = 0.0001;
fig(2, 4, 6);
for i = 1:nt
	% PC (adaptive filter) output
	z(i) = w1*p1(i)+w2*p2(i);
	
	% estimate of clean signal x
	xe(i) = y(i)+z(i);
	
	% error in estimate
	% caution: for display only - not a physically available signal
	e(i) = xe(i)-x(i);

	% covariance learning rule with
	% CF teaching signal = xe
	w1 = w1-beta*xe(i)*p1(i);
	w2 = w2-beta*xe(i)*p2(i);

	% occasional plots to show progress
	if (i > 500 & rem(i, 500) == 1) | i == nt
		subplot(3, 1, 1)
        stem([w1 w2]);
        axis([0.5 2.5 -1.1 1.1])
        title('Weights: w_1, w_2')
        set(gca, 'XTick', [1 2])
		
		subplot(3, 1, 2)
		plot(t, e.^2)
        xlim([t(1) t(end)]);
        title('Squared error e^2')	
        
        subplot(3, 1, 3)
        plot(t, x, t, xe, 'r');
        axis([t(i-500) t(i) -0.7 0.7]);
        title('Segment of signal and estimate')
		drawnow
	end
end

% Play mixture then cleaned signal
soundsc([y zeros(1, round(nt/10)) xe], fs);

%% Second pass using learned coeficients
for i = 1:nt
	% PC (adaptive filter) output
	z(i) = w1*p1(i)+w2*p2(i);
	
	% estimate of clean signal x
	xe(i) = y(i)+z(i);
	
	% error in estimate
	% caution: for display only - not a physically available signal
	e(i) = xe(i)-x(i);
end
fig(3, 6, 4)
plot(t, x, t, xe, 'r');
soundsc([y zeros(1, round(nt/10)) xe], fs);



