% Script SpectralBasis
% synthesise complex waveform using basis of sinusoids

%% set up 'spectral basis' to be triggered on parallel fibres

% set up time base
tmax = 1; nt = 1001;
t = linspace(0, tmax, nt)'; % column vector

f = 1/2;  % base frequency
nf = 50;  % number of harmonics
p = NaN(nt, nf);
for k = 1:nf
    p(:, k) = sin(2*pi*k*f*t); % k-th PF input in k-th coulumn of p
end

% plot sample of spectral basis
fig(1, 6, 4); clf

nplt = 8;
for k = 1:nplt
    subplot(nplt, 1, k)
    plot(t, p(:, k), 'LineWidth', 2.5); % axis off
    axis([0 1 -1.3 1.3])
    axis off
end
xlabel('time')


%% set up desired response
zd = double(t > 0.2 & t < 0.8)-0.5 *double(t > 0.35 & t < 0.657); % 'castle'
% zd = exp(-0.5*((t-0.5)/0.1).^2); % alternative bump response

% plot desired response
fig(2, 4, 4); clf
plot(t, zd);
axis([0 tmax -0.1 1.1])
title('Desired response')
xlabel('time')

%% Batch learning of PF/PC weights

w = zeros(nf, 1); % weights initially zero
beta = 0.01/nt;   % learning rate (normalised)
nbatch = 1000;    % number of presentations of batch
e = NaN(nt, 1);   % storage for teaching signal

for j = 1:nbatch
    % output of Purkinje cell
    z = p*w;

    % output error
    e = z-zd;

    % covariance learning
    w = w-beta*p'*e;
    
    % watch learning progress: comment out for efficiency
    fig(3, 6, 4); clf
    plot(t, zd, t, z)
    axis([0 tmax -0.1 1.1])
    title(sprintf('Batch %d of %d', j, nbatch))
    drawnow
end

%% Compare with optimal weights using batch least squares
wopt = p\zd;
zopt = p*wopt;

fig(4, 6, 4); clf
plot(t, zd, t, zopt, t, z)
title('Compare optimal and learned reconstruction')
xlabel('time')
legend('desired', 'optimal', 'learned', 'Location', 'best')

% figure(1);
% print -djpeg -r300 Fig4a
% 
% figure(2);
% print -djpeg -r300 Fig4b




